"""
Arthur Stroud,
Approximate Calculation of Multiple Integrals,
Prentice Hall, 1971.
"""
from ._stroud_secrest import stroud_secrest_07 as stroud_e3r_5_1
from ._stroud_secrest import stroud_secrest_08 as stroud_e3r_5_2
from ._stroud_secrest import stroud_secrest_09 as stroud_e3r_5_3
from ._stroud_secrest import stroud_secrest_10 as stroud_e3r_7_1
from ._stroud_secrest import stroud_secrest_11 as stroud_e3r_7_2

__all__ = [
    "stroud_e3r_5_1",
    "stroud_e3r_5_2",
    "stroud_e3r_5_3",
    "stroud_e3r_7_1",
    "stroud_e3r_7_2",
]
