import pathlib

from ...helpers import online
from .._helpers import _read, register

_source = online(
    authors=["Alvise Sommariva"],
    year="2012",
    url="https://www.math.unipd.it/~alvise/sets.html",
)

this_dir = pathlib.Path(__file__).resolve().parent


def sommariva_01():
    return _read(this_dir / "sommariva_01.json", _source)


def sommariva_02():
    return _read(this_dir / "sommariva_02.json", _source)


def sommariva_03():
    return _read(this_dir / "sommariva_03.json", _source)


def sommariva_04():
    return _read(this_dir / "sommariva_04.json", _source)


def sommariva_05():
    return _read(this_dir / "sommariva_05.json", _source)


def sommariva_06():
    return _read(this_dir / "sommariva_06.json", _source)


def sommariva_07():
    return _read(this_dir / "sommariva_07.json", _source)


def sommariva_08():
    return _read(this_dir / "sommariva_08.json", _source)


def sommariva_09():
    return _read(this_dir / "sommariva_09.json", _source)


def sommariva_10():
    return _read(this_dir / "sommariva_10.json", _source)


def sommariva_11():
    return _read(this_dir / "sommariva_11.json", _source)


def sommariva_12():
    return _read(this_dir / "sommariva_12.json", _source)


def sommariva_13():
    return _read(this_dir / "sommariva_13.json", _source)


def sommariva_14():
    return _read(this_dir / "sommariva_14.json", _source)


def sommariva_15():
    return _read(this_dir / "sommariva_15.json", _source)


def sommariva_16():
    return _read(this_dir / "sommariva_16.json", _source)


def sommariva_17():
    return _read(this_dir / "sommariva_17.json", _source)


def sommariva_18():
    return _read(this_dir / "sommariva_18.json", _source)


def sommariva_19():
    return _read(this_dir / "sommariva_19.json", _source)


def sommariva_20():
    return _read(this_dir / "sommariva_20.json", _source)


def sommariva_21():
    return _read(this_dir / "sommariva_21.json", _source)


def sommariva_22():
    return _read(this_dir / "sommariva_22.json", _source)


def sommariva_23():
    return _read(this_dir / "sommariva_23.json", _source)


def sommariva_24():
    return _read(this_dir / "sommariva_24.json", _source)


def sommariva_25():
    return _read(this_dir / "sommariva_25.json", _source)


def sommariva_26():
    return _read(this_dir / "sommariva_26.json", _source)


def sommariva_27():
    return _read(this_dir / "sommariva_27.json", _source)


def sommariva_28():
    return _read(this_dir / "sommariva_28.json", _source)


def sommariva_29():
    return _read(this_dir / "sommariva_29.json", _source)


def sommariva_30():
    return _read(this_dir / "sommariva_30.json", _source)


def sommariva_31():
    return _read(this_dir / "sommariva_31.json", _source)


def sommariva_32():
    return _read(this_dir / "sommariva_32.json", _source)


def sommariva_33():
    return _read(this_dir / "sommariva_33.json", _source)


def sommariva_34():
    return _read(this_dir / "sommariva_34.json", _source)


def sommariva_35():
    return _read(this_dir / "sommariva_35.json", _source)


def sommariva_36():
    return _read(this_dir / "sommariva_36.json", _source)


def sommariva_37():
    return _read(this_dir / "sommariva_37.json", _source)


def sommariva_38():
    return _read(this_dir / "sommariva_38.json", _source)


def sommariva_39():
    return _read(this_dir / "sommariva_39.json", _source)


def sommariva_40():
    return _read(this_dir / "sommariva_40.json", _source)


def sommariva_41():
    return _read(this_dir / "sommariva_41.json", _source)


def sommariva_42():
    return _read(this_dir / "sommariva_42.json", _source)


def sommariva_43():
    return _read(this_dir / "sommariva_43.json", _source)


def sommariva_44():
    return _read(this_dir / "sommariva_44.json", _source)


def sommariva_45():
    return _read(this_dir / "sommariva_45.json", _source)


def sommariva_46():
    return _read(this_dir / "sommariva_46.json", _source)


def sommariva_47():
    return _read(this_dir / "sommariva_47.json", _source)


def sommariva_48():
    return _read(this_dir / "sommariva_48.json", _source)


def sommariva_49():
    return _read(this_dir / "sommariva_49.json", _source)


def sommariva_50():
    return _read(this_dir / "sommariva_50.json", _source)


def sommariva_51():
    return _read(this_dir / "sommariva_51.json", _source)


def sommariva_52():
    return _read(this_dir / "sommariva_52.json", _source)


def sommariva_53():
    return _read(this_dir / "sommariva_53.json", _source)


def sommariva_54():
    return _read(this_dir / "sommariva_54.json", _source)


def sommariva_55():
    return _read(this_dir / "sommariva_55.json", _source)


register(
    [
        sommariva_01,
        sommariva_02,
        sommariva_03,
        sommariva_04,
        sommariva_05,
        sommariva_06,
        sommariva_07,
        sommariva_08,
        sommariva_09,
        sommariva_10,
        sommariva_11,
        sommariva_12,
        sommariva_13,
        sommariva_14,
        sommariva_15,
        sommariva_16,
        sommariva_17,
        sommariva_18,
        sommariva_19,
        sommariva_20,
        sommariva_21,
        sommariva_22,
        sommariva_23,
        sommariva_24,
        sommariva_25,
        sommariva_26,
        sommariva_27,
        sommariva_28,
        sommariva_29,
        sommariva_30,
        sommariva_31,
        sommariva_32,
        sommariva_33,
        sommariva_34,
        sommariva_35,
        sommariva_36,
        sommariva_37,
        sommariva_38,
        sommariva_39,
        sommariva_40,
        sommariva_41,
        sommariva_42,
        sommariva_43,
        sommariva_44,
        sommariva_45,
        sommariva_46,
        sommariva_47,
        sommariva_48,
        sommariva_49,
        sommariva_50,
        sommariva_51,
        sommariva_52,
        sommariva_53,
        sommariva_54,
        sommariva_55,
    ]
)
